@@ -15,6 +15,7 @@ import os
 import sys
 import time
 import base64
+import ipaddress
 from datetime import datetime
 import concurrent.futures
 from networksage_wrappers.wrappers import tests
@@ -39,6 +40,18 @@ def had_error(response):
     return False
 
 
+def get_destination_type(destination):
+    """Determines if a destination is a domain or an IP address
+    """
+    try:
+        if ipaddress.ip_address(destination):
+            return "IP"
+        else:
+            return "domain"
+    except:
+        return "domain"
+
+
 def upload_sample(sample_name, sample_data, sample_type, dns_data=None):
     """Upload a sample to the NetworkSage platform.
         + sample_name: whatever you want the sample to be named in the platform
@@ -190,6 +203,8 @@ def get_private_sample_metadata(uuid):
         return None
     result_json = json.loads(result.text)
     sample_metadata = result_json["body"]
+    if len(sample_metadata) == 0:
+        sample_metadata = None # not yet processed
     return sample_metadata
 
 
@@ -305,6 +320,78 @@ def get_global_count_for_secflow(secflow, uuid=None, is_public=False, session=No
     return count
 
 
+def get_subdomain_details_by_name(dest_and_port):
+    """Returns a list of Destinations (see https://www.seclarity.io/resources/glossary/ for details) for valid
+       subdomains of a given destination:port name when the destination is a domain. For any subdomain that does not
+       correspond to a known Destination, a dictionary containing only that subdomain's name and port will be returned
+       in the list. Note that this wrapper includes names for each subdomain, while the other destination calls do not.
+    """
+    destination = None
+    request_headers = { "apikey": api_key }
+
+    if ":" not in dest_and_port:
+        print("Error: Must include a port for destination.")
+        return None
+    parts = dest_and_port.split(":")
+    name = None
+    try:
+        if get_destination_type(parts[0]) == "domain":
+            name = parts[0]
+    except:
+        return None
+    if name is None:
+        print("Error: Must be a domain name.")
+        return None
+
+    subsets = name.split(".") # last one is the TLD or ccTLD
+    subdomains = set()
+    all_destinations = []
+    if len(subsets) == 2: # it's just example.com, so no subdomains to process
+        dest_dict = {"destinationName": dest_and_port}
+        dest_details = get_destination_by_name(dest_and_port)
+        dest_details.update(dest_dict)
+        all_destinations = [dest_details]
+    else:
+        for i in range(0, len(subsets)-1):
+            try:
+                subdomains.add(".".join(subsets[i:]) + ":" + parts[1])
+            except:
+                continue
+        result = []
+
+        session = requests.Session() # call all subdomains via one session
+        for subdomain in subdomains:
+            dest_dict = {"destinationName": subdomain}
+            dest_details = get_destination_by_name(subdomain, session=session)
+            dest_details.update(dest_dict)
+            all_destinations += [dest_details]
+    return all_destinations
+
+
+def get_destination_by_name(dest_and_port, session=None):
+    """Returns a Destination (see https://www.seclarity.io/resources/glossary/ for details) for a given
+        destination:port name. If no Destination exists, returns None. Session variable can be passed in if many
+        Destinations are being requested simultaneously (to reduce overhead).
+    """
+    destination = None
+    endpoint_url = "https://api.seclarity.io/sec/v1.0/destinations/" + dest_and_port
+    request_headers = { "apikey": api_key }
+
+    if ":" not in dest_and_port:
+        print("Error: Must include a port for destination.")
+        return None
+    if session is None:
+        result = requests.get(endpoint_url, headers=request_headers)
+    else:
+        result = session.get(endpoint_url, headers=request_headers)
+
+    if had_error(result):
+        return destination
+    result_json = json.loads(result.text)
+    destination = result_json["body"]
+    return destination
+
+
 def get_destination_for_secflow(secflow, uuid=None, is_public=False, session=None):
     """Returns a Destination (see https://www.seclarity.io/resources/glossary/
         for details) for a given Secflow. If no Destination exists, returns None. Session variable can be passed in if
